<?php

namespace Drupal\redirect_410\EventSubscriber;

use Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException;
use Drupal\Component\Plugin\Exception\PluginNotFoundException;
use Drupal\Core\Cache\CacheableResponse;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\PathProcessor\InboundPathProcessorInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\redirect\RedirectChecker;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpKernel\Event\RequestEvent;
use Symfony\Component\HttpKernel\KernelEvents;

/**
 * Redirect subscriber for controller requests.
 */
class RedirectRequestSubscriber implements EventSubscriberInterface {

  use StringTranslationTrait;

  /**
   * Constructs a Drupal\redirect_410\EventSubscriber object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager.
   * @param \Drupal\redirect\RedirectChecker $checker
   *   The redirect checker service.
   * @param \Drupal\Core\PathProcessor\InboundPathProcessorInterface $pathProcessor
   *   The path processor service.
   */
  public function __construct(
    protected EntityTypeManagerInterface $entityTypeManager,
    protected RedirectChecker $checker,
    protected InboundPathProcessorInterface $pathProcessor,
  ) {}

  /**
   * Handles the redirect if any found.
   *
   * @param \Symfony\Component\HttpKernel\Event\RequestEvent $event
   *   The event to process.
   */
  public function onKernelRequestCheckRedirect(RequestEvent $event): void {
    $request = clone $event->getRequest();

    if (!$this->checker->canRedirect($request)) {
      return;
    }

    if (\strpos($request->getPathInfo(), '/system/files/') === 0 && !$request->query->has('file')) {
      $path = $request->getPathInfo();
    }
    else {
      $path = $this->pathProcessor->processInbound($request->getPathInfo(), $request);
    }

    try {
      $storageHttpStatusEntity = $this->entityTypeManager->getStorage('http_status_entity');
    }
    catch (InvalidPluginDefinitionException $e) {
      return;
    }
    catch (PluginNotFoundException $e) {
      return;
    }

    $entities = $storageHttpStatusEntity->loadByProperties(['url' => $path]);
    if (!$httpStatus = \reset($entities)) {
      return;
    }
    $statusCode = $httpStatus->status_code;
    if ($statusCode !== '410') {
      return;
    }

    $content = $this->t('<h1>Gone</h1><p>The requested resource is no longer available on this server and there is no forwarding address.Please remove all references to this resource.</p>');
    $response = new CacheableResponse($content, $statusCode);
    $response->addCacheableDependency($httpStatus);
    $event->setResponse($response);
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    $events[KernelEvents::REQUEST][] = ['onKernelRequestCheckRedirect', 34];
    return $events;
  }

}
